import com.stepover.opensignatureapi.NativeServiceProvider;
import com.stepover.opensignatureapi.bridge.LibraryBridge;
import com.stepover.opensignatureapi.device.SoDeviceConfiguration;
import com.stepover.opensignatureapi.device.messages.ConfigurationMessage;
import com.stepover.opensignatureapi.device.nat.NativeDevice;
import com.stepover.opensignatureapi.helpers.Conversion;
import com.stepover.opensignatureapi.helpers.OpenSignatureException;
import com.stepover.opensignatureapi.signer.meta.SignatureBackgroundImage;
import com.stepover.opensignatureapi.signer.meta.SignatureColor;
import com.stepover.opensignatureapi.signer.task.SoDefaultSignTask;
import net.coobird.thumbnailator.Thumbnails;
import org.jopensignature.sign.*;
import purejavahidapi.HidDeviceInfo;
import sun.java2d.pipe.SpanShapeRenderer;

import javax.imageio.ImageIO;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.MemoryCacheImageInputStream;
import javax.swing.*;
import java.awt.*;
import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.Locale;
import java.util.concurrent.CountDownLatch;

public class SimpleSign {

    static boolean docsign = false;
    static boolean docview = false;

    public static void main(String[] args) {

        if (args.length > 0) {
            String mode = args[0];
            if (mode.equals("docsign")) {
                docsign = true;
            }
            else if( mode.equals("docview"))
            {
                docview = true;
            }
        }

        NativeServiceProvider serviceProvider = null;
        try
        {
            serviceProvider = new NativeServiceProvider();
        }
        catch( OpenSignatureException e )
        {
            e.printStackTrace();
        }

        SigningDevice[] devices = serviceProvider.getSigningDevices();

        if (devices.length == 0) {
            System.out.println("no device found");
            return;
        }

        SigningDevice signingDevice = null;
        for( SigningDevice hidDevice : devices )
        {
            HidDeviceInfo deviceInfo = ( (NativeDevice) hidDevice ).getHidDevice().getHidDeviceInfo();
            if( ( deviceInfo.getInterfaceNumber() != null && deviceInfo.getInterfaceNumber() == 0 ) )
            {
                signingDevice = hidDevice;
                break;
            }
        }

        SigningDeviceInfo signingDeviceInfo = signingDevice.getInfo( new Locale( "de" ) );
        System.out.println( "identifier: "  + signingDevice.getIdentifier() );
        System.out.println( "serial: "  + signingDeviceInfo.getSerialNumber() );
        System.out.println( "name: " + signingDeviceInfo.getName() );
        System.out.println( "firmware: " + signingDeviceInfo.getFirmwareVersion() );

        final JFrame frame = new JFrame();
        frame.setTitle("");
        frame.setLocation(100,100);
        frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);

        JPanel imagePanel = new JPanel();
        final JLabel imageLabel = new JLabel();
        imageLabel.setPreferredSize( new java.awt.Dimension( 690 / 2,
                184 / 2 ) );
        imagePanel.add( imageLabel );

        frame.getContentPane().add( BorderLayout.CENTER, imagePanel);
        frame.pack();

        frame.setVisible(true);

        final Signer signer = serviceProvider.createSigner( signingDevice );

        SoDeviceConfiguration config = new SoDeviceConfiguration();
        config.setPadmodeAfterSigning( SoDeviceConfiguration.PADMODE_MANUFACTURER_LOGO );

        config.setEmulationMode( false );
        if (docview) {
            SigningDevice finalSigningDevice = signingDevice;
            config.setStartSigningCallback( () -> startSigning(signer, finalSigningDevice, imageLabel ) );
        }

        config.setShowButtonsDuringSignature(!docview && !docsign ? false : true);

        signingDevice.send( new ConfigurationMessage( config ) );

        CountDownLatch countDownLatch = new CountDownLatch(1);

        if (docview||docsign) {

            try
            {
                ApplyDocumentRenderer applyDocumentRenderer = new ApplyDocumentRenderer( new FileInputStream( "test.pdf" ), docview?true:false );
                signingDevice.send( applyDocumentRenderer );
            }
            catch( FileNotFoundException e )
            {
                e.printStackTrace();
            }

        }

        if (!docview) {
            startSigning(signer, signingDevice, imageLabel);
        }

    }

    public static void startSigning( Signer signer, SigningDevice signingDevice, final JLabel imageLabel )
    {

        BufferedImage signImage = null;
        try
        {
            signImage = LibraryBridge.instance( signingDevice ).getSignImage();
            Graphics2D g2d = signImage.createGraphics();
            g2d.setColor( java.awt.Color.WHITE );
            g2d.fillRect( 0, 0, signImage.getWidth(), signImage.getHeight() );
            g2d.setPaint( java.awt.Color.BLACK );
            g2d.setFont(new Font("Serif", Font.BOLD, 20));
            String s = "Erika Mustermann";
            g2d.drawString(s, 30, signImage.getHeight());
            g2d.dispose();
        }catch (Exception e)
        {
            e.printStackTrace();
        }

        MetaInfoField[] si;
        if (!docsign && !docview) {
            si = new MetaInfoField[] {new SignatureBackgroundImage(signImage)};
        } else {
            si = new MetaInfoField[] {};
        }

        MemoryCacheImageInputStream docStream = null;

        try
        {
            docStream = new MemoryCacheImageInputStream( new FileInputStream( "test.pdf" ) );
        }
        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }

        org.jopensignature.sign.Rectangle signatureRectangle = new org.jopensignature.sign.Rectangle( 72, Conversion.mmToDots( 72, 70 ),
                Conversion.mmToDots( 72, 30 ),
                Conversion.mmToDots( 72,  70 ),
                Conversion.mmToDots( 72, 45 ) );

        SignatureContext signatureContext = new SignatureContext( docStream, si, 1, signatureRectangle, true );

        signer.startSigning( signatureContext, new SignatureListener()
        {

            public void updateRendering(final BufferedImage img)
            {
                SwingUtilities.invokeLater( new Runnable(){
                    public void run(){
                        try
                        {
                            imageLabel.setIcon( new ImageIcon( Thumbnails.of( img ).scale( 0.25 ).asBufferedImage() ) );
                        }
                        catch( IOException e )
                        {
                            e.printStackTrace();
                        }
                    }
                });

            }

            public void signatureFinished(SignatureResult result)
            {
                try
                {

                    if(result.getState().equals(SignatureResult.State.DONE))
                    {
                        System.out.println( "signature finished" );
                        ImageIO.write( result.getData().getSignatureImage(), "png", new File( "signature.png" ) );
                        InputStream inputStream = result.getData().getSignedDocumentStream();
                        if( inputStream != null )
                        {
                            File file = new File( "test_signed.pdf" );
                            FileOutputStream fileOutputStream = new FileOutputStream( file );
                            byte[] buffer = new byte[ 32 * 1024 ];
                            int bytesRead = 0;
                            while( (bytesRead = inputStream.read( buffer ) ) > 0 )
                            {
                                fileOutputStream.write( buffer, 0, bytesRead );
                            }

                            fileOutputStream.close();
                        }
                    }
                    else if(result.getState().equals(SignatureResult.State.CANCEL))
                    {
                        System.out.println("signature canceled");
                    }
                    else if(result.getState().equals(SignatureResult.State.FAILED))
                    {
                        System.out.println("signature failed");
                    }

                }
                catch( FileNotFoundException e )
                {
                    e.printStackTrace();
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }

                signingDevice.close();

            }

        } );

    }

}
